@@ -1,5 +1,6 @@
 from zope import interface
 from zope import component
+from zope.interface.interface import adapter_hooks
 
 from repoze.bfg.interfaces import IRequest
 from repoze.bfg.interfaces import INewRequest
@@ -8,12 +9,12 @@ from repoze.bfg.httprequest import global_registry, class_registry
 
 import urllib
 import operator
-import repoze.bfg.httprequest
-
-interfaces = repoze.bfg.httprequest.interfaces
 
 @interface.implementer(IRequest)
 def request_factory(environ):
+    import repoze.bfg.httprequest
+    interfaces = repoze.bfg.httprequest.interfaces
+    
     class Interface(type(interface.Interface)):
         def __repr__(self):
             return '<IHTTPRequest %s>' % urllib.urlencode(environ)
@@ -21,9 +22,8 @@ def request_factory(environ):
     IHTTPRequest = Interface("IHTTPRequest", (IRequest,))
         
     ifaces = []
-    
     for name, value in environ.items():
-        key = (name, value)
+        key = (name, str(value).lower())
         iface = global_registry.get(key)
         if iface is None:
             class IMarker(interface.Interface):
@@ -48,15 +48,22 @@ def request_factory(environ):
 
     return iface
 
-# we make this adapter available at module import time
-component.provideAdapter(request_factory, (dict,))
+# add adapter hook to adapt dicts to the ``IRequest`` interface
+def adapt_dict_to_request(iface, ob):
+    if iface is IRequest:
+        return request_factory(ob)
+    
+adapter_hooks.append(adapt_dict_to_request)
 
 @component.adapter(INewRequest)
 def new_request_handler(new_request_event):
     ifaces = []
     request = new_request_event.request
     for name1, value1 in request.environ.items():
+        name1 = name1.lower()
+        value1 = str(value1).lower()
         for (name2, value2), iface in global_registry.items():
+            name2 = name2.lower()
             if name1 == name2 and value2 in value1:
                 ifaces.append(iface)
 
